#include <motelib/system.h>
#include <motelib/timer.h>
#include <motelib/actor.h>
#include <motelib/radio.h>
#include <motelib/uart.h>

#include "pwm.h"
#include "messages.h"
#include "sharedvar.h"

#define BASE_ADDR 0

#define SERVO_HORIZONTAL TIMER0_OUTPUT
#define SERVO_VERTICAL 	 TIMER1_OUTPUT

#define LED_RED 	TIMER2_OUTPUT
#define LED_GREEN TIMER3_OUTPUT
#define LED_BLUE 	TIMER4_OUTPUT

#define PIR_INPUT ACTOR_14

#define LED_PERIOD 2500
#define LED_MULTIPLIER 100

#define SERVO_PERIOD 20000
#define MAX_DEGREE 2100
#define MIN_DEGREE 500
#define SERVO_STEP 20

enum{SERVO_H_PWM, SERVO_V_PWM,
	  LED_RED_PWM, LED_GREEN_PWM, LED_BLUE_PWM};

Timer pir_t;
uint8_t pir_state;
uint16_t servo_h_degree, servo_v_degree;
StationStatus status;

void readPIR(Timer* t);
void detectPIR();

void lightSetState();
void lightSetBrightness(uint8_t bn);
void lightSetColor(uint8_t color);
void lightSetDirection(uint8_t dir);

void rxHandler(Address source, MessageType type, void *message, uint8_t len);

void readPIR(Timer* t)
{
	if(pir_state == 0)
	{
		if(actorGetState(PIR_INPUT) == 1)
		{
			pir_state = 1;
			detectPIR();
		}
	}
	else if(pir_state == 1)
	{
		if(actorGetState(PIR_INPUT) == 0)
			pir_state = 0;
	}
}

void detectPIR()
{
	radioRequestTx(BASE_ADDR,
						TYPE_PIR_DETECTION,
						NULL,
						0,
						NULL);
}

void sendStationStatus()
{
	radioRequestTx(BASE_ADDR,
						TYPE_STATUS_REPORT,
						&status,
						sizeof(StationStatus),
						NULL);
}

void lightSetState()
{	
	printf_jennic("\r\nState = %d, Color = %d, Bn = %d\r\n",
						status.enable, status.color, status.brightness);
	
	pwmStop(LED_RED_PWM);
	pwmStop(LED_GREEN_PWM);
	pwmStop(LED_BLUE_PWM);
	
	if(status.enable == 0)
	{
		pwmStart(LED_RED_PWM, MIN_BRIGHTNESS,
					LED_PERIOD, PERIODIC);
		pwmStart(LED_GREEN_PWM, MIN_BRIGHTNESS,
					LED_PERIOD, PERIODIC);
		pwmStart(LED_BLUE_PWM, MIN_BRIGHTNESS,
					LED_PERIOD, PERIODIC);
	}
	else if(status.enable == 1)
	{
		if(status.color == RED)
		{
			pwmStart(LED_RED_PWM,status.brightness*LED_MULTIPLIER,
						LED_PERIOD, PERIODIC);
			pwmStart(LED_GREEN_PWM, MIN_BRIGHTNESS,
						LED_PERIOD, PERIODIC);
			pwmStart(LED_BLUE_PWM, MIN_BRIGHTNESS,
						LED_PERIOD, PERIODIC);
		}
		else if(status.color == GREEN)
		{
			pwmStart(LED_GREEN_PWM, status.brightness*LED_MULTIPLIER,
						LED_PERIOD, PERIODIC);
			pwmStart(LED_RED_PWM, MIN_BRIGHTNESS,
						LED_PERIOD, PERIODIC);
			pwmStart(LED_BLUE_PWM, MIN_BRIGHTNESS,
						LED_PERIOD, PERIODIC);
		}
		else if(status.color == WHITE)
		{
			pwmStart(LED_RED_PWM, status.brightness*LED_MULTIPLIER,
						LED_PERIOD, PERIODIC);
			pwmStart(LED_GREEN_PWM, status.brightness*LED_MULTIPLIER,
						LED_PERIOD, PERIODIC);
			pwmStart(LED_BLUE_PWM, status.brightness*LED_MULTIPLIER,
						LED_PERIOD, PERIODIC);
		}
	}
}

void lightSetBrightness(uint8_t bn)
{
	status.brightness = bn;
	lightSetState();
}

void lightSetColor(uint8_t color)
{
	status.color = color;
	lightSetState();
}

void lightSetDirection(uint8_t dir)
{
	pwmStop(SERVO_H_PWM);
	pwmStop(SERVO_V_PWM);

	if(dir == LEFT && servo_h_degree > MIN_DEGREE)
	{
		servo_h_degree -= SERVO_STEP;
		if(servo_h_degree < MIN_DEGREE)
			servo_h_degree = MIN_DEGREE;
	}
	else if(dir == RIGHT && servo_h_degree < MAX_DEGREE)
	{
		servo_h_degree += SERVO_STEP;
		if(servo_h_degree > MAX_DEGREE)
			servo_h_degree = MAX_DEGREE;
	}
	else if(dir == UP && servo_v_degree < MAX_DEGREE)
	{
		servo_v_degree += SERVO_STEP;
		if(servo_v_degree > MAX_DEGREE)
			servo_v_degree = MAX_DEGREE;
	}
	else if(dir == DOWN && servo_v_degree > MIN_DEGREE)
	{
		servo_v_degree -= SERVO_STEP;
		if(servo_v_degree < MIN_DEGREE)
			servo_v_degree = MIN_DEGREE;
	}

	//printf_jennic("\r\nH_degree = %4d, V_degree = %4d\r\n",
	//				  servo_h_degree, servo_v_degree);

	pwmStart(SERVO_H_PWM, servo_h_degree, SERVO_PERIOD, PERIODIC);
	pwmStart(SERVO_V_PWM, servo_v_degree, SERVO_PERIOD, PERIODIC);
}

void rxHandler(Address source, MessageType type, void *message, uint8_t len)
{
	printf_jennic("Received pkt from Node ID %d,"
					  "len = %d, type = %d\r\n",
					  source, len, type);

	if(type == TYPE_LIGHT_ENABLE)
	{
		status.enable = *((uint8_t*)message);
		lightSetState();
	}
	else if(type == TYPE_DIMMER)
	{
		if(status.enable == 1)
			lightSetBrightness(*((uint8_t*)message));
	}
	else if(type == TYPE_LIGHT_COLOR)
	{
		if(status.enable == 1)
			lightSetColor(*((uint8_t*)message));
	}
	else if(type == TYPE_LIGHT_DIRECTION)
	{
		lightSetDirection(*((uint8_t*)message));
	}
	else if(type == TYPE_REQUEST_STATUS)
	{
		sendStationStatus();
	}
}

void boot()
{
	uartEnable(true, true);
	radioSetRxHandler(rxHandler);

	pir_state = 0;
	status.enable = 0;
	status.color = RED;
	status.brightness = MAX_BRIGHTNESS/2;

	servo_h_degree = (MAX_DEGREE+MIN_DEGREE)/2;
	servo_v_degree = (MAX_DEGREE+MIN_DEGREE)/2;

	pwmCreate(5);
	
	pwmStart(LED_RED_PWM, MIN_BRIGHTNESS,
				LED_PERIOD, PERIODIC);
	pwmStart(LED_GREEN_PWM, MIN_BRIGHTNESS,
				LED_PERIOD, PERIODIC);
	pwmStart(LED_BLUE_PWM, MIN_BRIGHTNESS,
				LED_PERIOD, PERIODIC);
	
	//Set Light to center direction
	pwmStart(SERVO_H_PWM, servo_h_degree, SERVO_PERIOD, PERIODIC);
	pwmStart(SERVO_V_PWM, servo_v_degree, SERVO_PERIOD, PERIODIC);

	timerCreate(&pir_t);
	timerStart(&pir_t, TIMER_PERIODIC, 100, readPIR);
}
